@@ -1,4 +1,4 @@
-#ident "$Id: mkzftree.h,v 1.3 2001/07/31 03:20:52 hpa Exp $"
+#ident "$Id: mkzftree.h,v 1.4 2002/11/07 03:58:18 hpa Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *   Copyright 2001 H. Peter Anvin - All Rights Reserved
@@ -49,6 +49,7 @@ enum verbosity {		/* Message verbosity */
   vl_quiet,			/* No messages */
   vl_error,			/* Error messages only */
   vl_filename,			/* Display filenames */
+  vl_crib,			/* Cribbing files */
 };
 #define default_verbosity vl_error
 struct cmdline_options {
@@ -66,8 +67,8 @@ struct cmdline_options {
 extern struct cmdline_options opt;
 
 /* walk.c */
-int munge_tree(const char *, const char *);
-int munge_entry(const char *, const char *, const struct stat *);
+int munge_tree(const char *, const char *, const char *);
+int munge_entry(const char *, const char *, const char *, const struct stat *);
 
 /* workers.c */
 void wait_for_all_workers(void);
