@@ -1,7 +1,7 @@
-#ident "$Id: walk.c,v 1.4 2001/07/31 03:23:33 hpa Exp $"
+#ident "$Id: walk.c,v 1.5 2002/11/07 03:58:18 hpa Exp $"
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 2001 H. Peter Anvin - All Rights Reserved
+ *   Copyright 2001-2002 H. Peter Anvin - All Rights Reserved
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -28,13 +28,53 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include "mkzftree.h"
+#include "iso9660.h"
 
-static int munge_path(const char *inpath, const char *outpath, struct stat *st)
+static int munge_file(const char *inpath, const char *outpath,
+		      const char *cribpath, struct stat *st)
 {
   FILE *in, *out;
   int err = 0;
   struct utimbuf ut;
 
+  if ( cribpath ) {
+    struct stat cst;
+    struct compressed_file_header cfh;
+
+    /* Compare as much as we realistically can */
+    if ( !stat(cribpath, &cst) &&
+	 st->st_mode == cst.st_mode &&
+	 st->st_uid == cst.st_uid &&
+	 st->st_gid == cst.st_gid &&
+	 st->st_mtime == cst.st_mtime ) {
+      if ( (in = fopen(cribpath, "rb")) ) {
+	int e = fread(&cfh, 1, sizeof cfh, in);
+	fclose(in);
+	/* Attempt to restore the atime */
+	ut.actime  = cst.st_atime;
+	ut.modtime = cst.st_mtime;
+	utime(cribpath, &ut);
+
+	if ( (e == sizeof cfh &&
+	      !memcmp(cfh.magic, zisofs_magic, sizeof zisofs_magic) &&
+	      (off_t)get_731(cfh.uncompressed_len) == st->st_size) ||
+	     (st->st_size == cst.st_size &&
+	      (e < (int)(sizeof zisofs_magic) ||
+	       memcmp(cfh.magic, zisofs_magic, sizeof zisofs_magic))) ) {
+	  /* File is cribbable.  Steal it. */
+	  if ( !link(cribpath, outpath) ) {
+	    message(vl_crib, "crib: %s -> %s\n", cribpath, outpath);
+	    ut.actime  = st->st_atime;
+	    ut.modtime = st->st_mtime;
+	    utime(outpath, &ut);	/* Set the the atime */
+	    
+	    return 0;
+	  }
+	}
+      }
+    }
+  }
+
   in = fopen(inpath, "rb");
   if ( !in )
     return EX_NOINPUT;
@@ -49,19 +89,12 @@ static int munge_path(const char *inpath, const char *outpath, struct stat *st)
     fclose(in);
     fclose(out);
     
-#ifdef HAVE_LCHOWN
-    lchown(outpath, st->st_uid, st->st_gid);
-#endif
-    if ( !S_ISLNK(st->st_mode) ) {
-#ifndef HAVE_LCHOWN
-      chown(outpath, st->st_uid, st->st_gid);
-#endif
-      chmod(outpath, st->st_mode);
-      ut.actime  = st->st_atime;
-      ut.modtime = st->st_mtime;
-      utime(outpath, &ut);
-    }
-
+    chown(outpath, st->st_uid, st->st_gid);
+    chmod(outpath, st->st_mode);
+    ut.actime  = st->st_atime;
+    ut.modtime = st->st_mtime;
+    utime(outpath, &ut);
+    
     end_worker(err);
   } else {
     fclose(in);
@@ -71,9 +104,10 @@ static int munge_path(const char *inpath, const char *outpath, struct stat *st)
   return err;
 }
 
-int munge_tree(const char *intree, const char *outtree)
+int munge_tree(const char *intree, const char *outtree, const char *cribtree)
 {
-  char *in_path, *out_path, *in_file, *out_file;
+  char *in_path, *out_path, *crib_path;
+  char *in_file, *out_file, *crib_file;
   DIR *thisdir;
   struct dirent *dirent;
   struct stat dirst;
@@ -82,17 +116,24 @@ int munge_tree(const char *intree, const char *outtree)
   /* Construct buffers with the common filename prefix, and point to the end */
 
   in_path = xmalloc(strlen(intree) + NAME_MAX + 2);
-  out_path = xmalloc(strlen(outtree) + NAME_MAX + 2);
-
   strcpy(in_path, intree);
-  strcpy(out_path, outtree);
-
   in_file = strchr(in_path, '\0');
-  out_file = strchr(out_path, '\0');
-
   *in_file++ = '/';
+
+  out_path = xmalloc(strlen(outtree) + NAME_MAX + 2);
+  strcpy(out_path, outtree);
+  out_file = strchr(out_path, '\0');
   *out_file++ = '/';
 
+  if ( cribtree ) {
+    crib_path = xmalloc(strlen(cribtree) + NAME_MAX + 2);
+    strcpy(crib_path, cribtree);
+    crib_file = strchr(crib_path, '\0');
+    *crib_file++ = '/';
+  } else {
+    crib_path = crib_file = NULL;
+  }
+
   /* Get directory information */
   if ( stat(intree, &dirst) ) {
     message(vl_error, "%s: Failed to stat directory %s: %s\n",
@@ -122,8 +163,10 @@ int munge_tree(const char *intree, const char *outtree)
     
     strcpy(in_file, dirent->d_name);
     strcpy(out_file, dirent->d_name);
+    if ( crib_file )
+      strcpy(crib_file, dirent->d_name);
     
-    err = munge_entry(in_path, out_path, &dirst);
+    err = munge_entry(in_path, out_path, crib_path, &dirst);
     if ( err )
       break;
   }
@@ -136,7 +179,8 @@ int munge_tree(const char *intree, const char *outtree)
 }
 
 
-int munge_entry(const char *in_path, const char *out_path, const struct stat *dirst)
+int munge_entry(const char *in_path, const char *out_path,
+		const char *crib_path, const struct stat *dirst)
 {
   struct stat st;
   struct utimbuf ut;
@@ -165,7 +209,7 @@ int munge_entry(const char *in_path, const char *out_path, const struct stat *di
 	}
       } else {
 	/* First encounter, compress and enter into hash */
-	if ( (err = munge_path(in_path, out_path, &st)) != 0 ) {
+	if ( (err = munge_file(in_path, out_path, crib_path, &st)) != 0 ) {
 	  message(vl_error, "%s: %s: %s", program, in_path, strerror(errno));
 	  return err;
 	}
@@ -173,7 +217,7 @@ int munge_entry(const char *in_path, const char *out_path, const struct stat *di
       }
     } else {
       /* Singleton file; no funnies */
-      if ( (err = munge_path(in_path, out_path, &st)) != 0 ) {
+      if ( (err = munge_file(in_path, out_path, crib_path, &st)) != 0 ) {
 	message(vl_error, "%s: %s: %s", program, in_path, strerror(errno));
 	return err;
       }
@@ -182,7 +226,7 @@ int munge_entry(const char *in_path, const char *out_path, const struct stat *di
     /* Recursion: see recursion */
     if ( !opt.onedir &&
 	 (!opt.onefs || (dirst && dirst->st_dev == st.st_dev)) ) {
-      if ( (err = munge_tree(in_path, out_path)) != 0 )
+      if ( (err = munge_tree(in_path, out_path, crib_path)) != 0 )
 	return err;
     } else if ( opt.do_mkdir ) {
       /* Create stub directories */
@@ -244,7 +288,7 @@ int munge_entry(const char *in_path, const char *out_path, const struct stat *di
     }
   }
   
-  /* This is done by munge_path() for files */
+  /* This is done by munge_file() for files */
   if ( !S_ISREG(st.st_mode) ) {
 #ifdef HAVE_LCHOWN
     if ( lchown(out_path, st.st_uid, st.st_gid) && opt.sloppy && !err ) {
